import re
from sub import Sub, RenderTask, ParsedValue

class PythonPackgramSub(Sub):
    
    #Public
    
    render_setup = RenderTask('setup.tpl', 'setup.py')
    
    author = ParsedValue(
        file_pattern='AUTHORS.rst', 
        text_pattern=r'\n-\s+(.*?)\s+<.*?@.*?>',
        processors=[
            lambda items: items[0],
        ],
    ) 
         
    author_email = ParsedValue(
        file_pattern='AUTHORS.rst',
        text_pattern=r'\n-\s+.*?\s+<(.*?@.*?)>',    
        processors=[
            lambda items: items[0],
        ],
    )   
    
    classifiers = ParsedValue(
        file_pattern='README.rst',
        text_pattern=r'- ([^\n]* :: [^\n]*)',       
    )
    
    description = ParsedValue(
        file_pattern='README.rst',
        text_pattern=r'.*?=\s+(.*?)(?:\n|$)',    
        processors=[
            lambda items: items[0],
        ],
    )
    
    license = ParsedValue(
        file_pattern='LICENSE.rst',
        text_pattern=r'(.*?)\s+=',
        processors=[
            lambda items: items[0],
        ],
    )
    
    long_description = ParsedValue(
        file_pattern='README.rst',
        text_pattern=r'(.*)',
        text_pattern_flags=re.DOTALL,   
        processors=[
            lambda items: items[0],
        ],
    )    
     
    maintainer = ParsedValue(
        file_pattern='AUTHORS.rst',
        text_pattern=r'\n-\s+(.*?)\s+<.*?@.*?>',  
        processors=[
            lambda items: items[0],
        ],
    ) 
       
    maintainer_email = ParsedValue(
        file_pattern='AUTHORS.rst',
        text_pattern=r'\n-\s+.*?\s+<(.*?@.*?)>',   
        processors=[
            lambda items: items[0],
        ],
    )
       
    name = ParsedValue(
        file_pattern='README.rst',
        text_pattern=r'(.*?)\s+=',
        processors=[
            lambda items: items[0],
            str.lower,
        ],
    )
    
    version = ParsedValue(
        file_pattern='.*version.py',
        text_pattern=r'(?:major|minor|micro|level)\s*=\s*[\'"]?(\w*)[\'"]?',
        processors=[
            lambda items: [item for item in items if item != 'final'],
            lambda items: '.'.join(items),
        ],
    )                        